function [mu_mt_out,W] = MTA_pairwise(mu_st,s_st,N,gamma,A)

%% MTA function
% inputs: 
% mu_st - single task means
% A - TxT sim matrix
% N - vector or scalar of number of samples in each task. non-negative integer
% gamma - regularization factor. positive real scalar (if gamma = 0, W = I,
% so no need to call this function) 

T = size(mu_st,1);

% setting some defaults
if nargin < 4;    gamma = 1;        end
if nargin < 3;    N = ones(T,1);    end
if length(s_st) == 1;
    s_st = s_st*ones(T,1);
end

% only doing MTA (initially) for the tasks where N > 0
Sigma_diag = s_st./N;
prob_flag = ~isfinite(Sigma_diag); % we have some issue here

if T > 1 
    % scaling gamma with number of tasks
    gamma = gamma/T;

    % making AS, the symmetrized version of A
    if nargin < 5
        dists_sq = distsAndSims(mu_st);
        zero_flag = (dists_sq == 0);
        dists_sq(zero_flag) = min(min(dists_sq(~zero_flag))); % getting rid of zeros
        A = 2./dists_sq;
        A = gamma*A;
    else
        A = gamma*((A+A')/2);
    end

    % making Ls, the unnormalized graph laplacian of AS
    diagonal_inds = 1:T+1:T^2; % entries of the diagonal 
    A(diagonal_inds) = 0; % zeroing out diagonal
    d = sum(A,2); % sum of the rows
    L = -A; % the off-diagonal values of L are negative values of AS
    L(diagonal_inds) = d; % only true if diagonal is zero!

    % now have to left multiply L by S.
    % this is equivalent to multiplying the t-th row by s_st(t)/N(t)
    B = bsxfun(@times,L(~prob_flag,~prob_flag),Sigma_diag(~prob_flag)); % = Sigma*LS if and only if N is a column vector
%     B = LS(~zero_flag,~zero_flag);
    
    % the final B is identity plus the previous step
    % now we have a submatrix because of the zero_flag
    T = sum(~prob_flag); 
    diagonal_inds = 1:T+1:T^2;
    B(diagonal_inds) = B(diagonal_inds) + 1;
 
    % calculating mu_mt and W (optionally)
    if nargout > 1;
        W = inv(B); 
        mu_mt = W*mu_st(~prob_flag,:);
    else
        W = [];
        mu_mt = B\mu_st(~prob_flag,:); % should be equivalent to inv(B)*m_st
    end

    % if problems came up, go the expensive route
    if any(~isfinite(mu_mt(:))) 
        keyboard
       W = pinv(B);  
       mu_mt = pinv(B)*mu_st(~prob_flag,:);
    end

    % the non-zero N(t) entries with multi-task estimates,
    % and the rest are single-task estimates
    if sum(prob_flag) > 0 
        LS_lu = L(prob_flag,~prob_flag);
        LS_ll = L(prob_flag,prob_flag);
        mu_l = LS_ll\(-LS_lu*mu_mt);
        mu_mt_out(prob_flag,:) = mu_l; % does not depend on mu_st!
        % the others are multi-task versions
        mu_mt_out(~prob_flag,:) = mu_mt;
        % if any are not finite, just set them to the single-task version
        not_finite = ~isfinite(mu_mt_out);
        mu_mt_out(not_finite) = mu_st(not_finite);
    else
        mu_mt_out = mu_mt;
    end

else % if only one task is legit, just return the input, 
        % and don't bother doing anything

    mu_mt_out = mu_st;
    W = 1;

end

