%% This function loads the grades dataset and runs the different estimators.
%% See Feldman et al. 2014 for details.

clear all

%% loading data
load('grades_sorted_by_class_size.mat')
final_grades = final_grades_sorted;
hwk_grades = hwk_grades_sorted;

%% Regularization parameter choices for cross-val.
pool_flag = 1;
gammas = 2.^(-15:1:15);
lambdas = gammas./(gammas+1); % if gamma = lambda/(1-lambda), then lambda = gamma/(1+gamma)

'Running, should take less than 5 min.'

% Each class is treated as an independent experiment.
for i = 1:length(final_grades)
    
    
    %% remove any zero-variance tasks, initialize
    s = var(hwk_grades{i},[],2);
    X = num2cell(hwk_grades{i}(s>0,:),2);
    final_grades{i} = final_grades{i}(s>0);
    
    % N is a vector with num_task components and each component is
    % the number of samples (= hwk grade) for each task (= student)
    clear num_samples_per_task;
    for j = 1:length(X)
        num_samples_per_task(j) = length(X{j});
    end
    
    % Each student in the i-th class is treated as an independent task.
    num_tasks = length(final_grades{i});
    
    %% single-task estimates
    [mu_st, s_st, mu_one, s_pool] = get_st_and_pooled_stats(X);
    if pool_flag == 1; s_st = s_pool; end
        
    %% MTA estimates
    [mu_js, mu_const, mu_minimax] = run_MTA_suite(mu_st, s_st, num_samples_per_task);
    
    %% MTA CV estimates - this does LOOCV to get the best params.
    params_opt = MTA_CV_LOOCV(X, num_samples_per_task, gammas, lambdas, pool_flag);
    % Run standard suite of various convex combinations using the best
    % params determined by the above leave-one-out cv
    [mu_const_CV,mu_minimax_CV,mu_am_CV] = run_MTA_suite_CV(mu_st',s_st',num_samples_per_task',params_opt);
    
    
    %% errors
    err(1,i) = errors(mu_st,final_grades{i});
    err(2,i) = errors(mu_one,final_grades{i});
    
    err(3,i) = errors(mu_js,final_grades{i});
    err(4,i) = errors(mu_am_CV',final_grades{i});
    
    err(5,i) = errors(mu_const,final_grades{i});
    err(6,i) = errors(mu_const_CV,final_grades{i});
    
    err(7,i) = errors(mu_minimax,final_grades{i});
    err(8,i) = errors(mu_minimax_CV,final_grades{i});
end

%% These are the percent gains over the sample means for each method.
prct_2 = 100*(err - repmat(err(1,:),[size(err,1) 1]))./repmat(err(1,:),[size(err,1) 1]);

%% Here are the average gains and the std dev of the gains.
prct_stats = round([mean(prct_2'); std(prct_2')]*10)/10;

prct_2 = round(prct_2'*10)/10;

'Here is the latex that prints out the table in the Feldman 2014 paper.'
latex([prct_2(:,2:end); prct_stats(:,2:end)])

